package com.iot.modbus_rtcp.jobs;

import cn.hutool.core.map.MapUtil;
import com.iot.modbus_rtcp.controller.ModbusTCPController;
import com.iot.modbus_rtcp.dto.CommandTypeComparable;
import com.iot.modbus_rtcp.dto.ModbusCommandDto;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 18:33
 */
@Component
@RequiredArgsConstructor
public class AutoCollectJobs {

    private static final String SQL = "SELECT ref_id as deviceId, command, message_length messageLength FROM commands " +
            "WHERE type = 'COLLECTION' and ref_type = 'DEVICE' and id < 10099 order by id";

    private final JdbcTemplate jdbcTemplate;
    private final ModbusTCPController controller;

    @Scheduled(cron = "0/30 * * * * ? ")
    public void autoCollect() {
        int pageIndex = 0;
        int pageSize = 1000;
        long timestamp = System.currentTimeMillis();
        List<Map<String, Object>> resultList = null;
        while (Objects.isNull(resultList) || resultList.size() >= pageSize) {
            resultList = this.jdbcTemplate.queryForList(SQL + " LIMIT " + (pageIndex++) + "," + pageSize);
            if (ObjectUtils.isNotEmpty(resultList)) {
                this.controller.collect(resultList.stream()
                        .filter(item -> StringUtils.isNotEmpty(MapUtil.getStr(item, "identifier")))
                        .map(item -> ModbusCommandDto.builder()
                                .command(MapUtil.getStr(item, "command"))
                                .identifier("4B454E454E4731343030303030333538")
                                .length(MapUtil.getInt(item, "messageLength"))
                                .type(CommandTypeComparable.CommandType.COLLECTION)
                                .key("4B454E454E4731343030303030333538/" + MapUtil.getStr(item, "deviceId") + "/" + timestamp)
                                .build())
                        .collect(Collectors.toList()));
            }
        }

    }
}
